@@ -19,14 +19,21 @@ from optax._src.alias import adabelief
 from optax._src.alias import adafactor
 from optax._src.alias import adagrad
 from optax._src.alias import adam
+from optax._src.alias import adamax
+from optax._src.alias import adamaxw
 from optax._src.alias import adamw
+from optax._src.alias import amsgrad
 from optax._src.alias import dpsgd
 from optax._src.alias import fromage
 from optax._src.alias import lamb
 from optax._src.alias import lars
+from optax._src.alias import MaskOrFn
 from optax._src.alias import noisy_sgd
+from optax._src.alias import novograd
+from optax._src.alias import optimistic_gradient_descent
 from optax._src.alias import radam
 from optax._src.alias import rmsprop
+from optax._src.alias import ScalarOrSchedule
 from optax._src.alias import sgd
 from optax._src.alias import sm3
 from optax._src.alias import yogi
@@ -72,6 +79,7 @@ from optax._src.loss import cosine_distance
 from optax._src.loss import cosine_similarity
 from optax._src.loss import ctc_loss
 from optax._src.loss import ctc_loss_with_forward_probs
+from optax._src.loss import hinge_loss
 from optax._src.loss import huber_loss
 from optax._src.loss import l2_loss
 from optax._src.loss import log_cosh
@@ -119,7 +127,11 @@ from optax._src.transform import ema
 from optax._src.transform import EmaState
 from optax._src.transform import scale
 from optax._src.transform import scale_by_adam
+from optax._src.transform import scale_by_adamax
+from optax._src.transform import scale_by_amsgrad
 from optax._src.transform import scale_by_belief
+from optax._src.transform import scale_by_novograd
+from optax._src.transform import scale_by_optimistic_gradient
 from optax._src.transform import scale_by_param_block_norm
 from optax._src.transform import scale_by_param_block_rms
 from optax._src.transform import scale_by_radam
@@ -131,8 +143,10 @@ from optax._src.transform import scale_by_stddev
 from optax._src.transform import scale_by_trust_ratio
 from optax._src.transform import scale_by_yogi
 from optax._src.transform import ScaleByAdamState
+from optax._src.transform import ScaleByAmsgradState
 from optax._src.transform import ScaleByBeliefState
 from optax._src.transform import ScaleByFromageState
+from optax._src.transform import ScaleByNovogradState
 from optax._src.transform import ScaleByRmsState
 from optax._src.transform import ScaleByRssState
 from optax._src.transform import ScaleByRStdDevState
@@ -164,13 +178,15 @@ from optax._src.wrappers import ShouldSkipUpdateFunction
 from optax._src.wrappers import skip_large_updates
 from optax._src.wrappers import skip_not_finite
 
-__version__ = "0.1.3"
+__version__ = "0.1.4"
 
 __all__ = (
     "adabelief",
     "adafactor",
     "adagrad",
     "adam",
+    "adamax",
+    "adamaxw",
     "adamw",
     "adaptive_grad_clip",
     "AdaptiveGradClipState",
@@ -180,6 +196,7 @@ __all__ = (
     "additive_weight_decay",
     "AdditiveWeightDecayState",
     "AddNoiseState",
+    "amsgrad",
     "apply_every",
     "apply_if_finite",
     "apply_updates",
@@ -214,6 +231,7 @@ __all__ = (
     "fromage",
     "global_norm",
     "GradientTransformation",
+    "hinge_loss",
     "hessian_diag",
     "huber_loss",
     "hvp",
@@ -233,6 +251,7 @@ __all__ = (
     "LookaheadParams",
     "LookaheadState",
     "masked",
+    "MaskOrFn",
     "MaskedState",
     "matrix_inverse_pth_root",
     "maybe_update",
@@ -245,6 +264,7 @@ __all__ = (
     "MultiStepsState",
     "MultiTransformState",
     "noisy_sgd",
+    "novograd",
     "NonNegativeParamsState",
     "OptState",
     "Params",
@@ -260,9 +280,13 @@ __all__ = (
     "safe_int32_increment",
     "safe_norm",
     "safe_root_mean_squares",
+    "ScalarOrSchedule",
     "scale_by_adam",
+    "scale_by_adamax",
+    "scale_by_amsgrad",
     "scale_by_belief",
     "scale_by_factored_rms",
+    "scale_by_novograd",
     "scale_by_param_block_norm",
     "scale_by_param_block_rms",
     "scale_by_radam",
@@ -276,8 +300,10 @@ __all__ = (
     "scale_gradient",
     "scale",
     "ScaleByAdamState",
+    "ScaleByAmsgradState",
     "ScaleByBeliefState",
     "ScaleByFromageState",
+    "ScaleByNovogradState",
     "ScaleByRmsState",
     "ScaleByRssState",
     "ScaleByRStdDevState",
