@@ -44,7 +44,7 @@ class ClippingTest(absltest.TestCase):
     clipper = clipping.clip(1.)
     clipped_updates, _ = clipper.update(updates, None)
     chex.assert_tree_all_close(
-        clipped_updates, jax.tree_map(jnp.ones_like, updates))
+        clipped_updates, jax.tree_util.tree_map(jnp.ones_like, updates))
 
   def test_clip_by_block_rms(self):
     rmf_fn = lambda t: jnp.sqrt(jnp.mean(t**2))
@@ -80,10 +80,11 @@ class ClippingTest(absltest.TestCase):
 
       # Check that the clipper actually works and upd_norm is < c * param_norm.
       updates, _ = clipper.update(updates, None, params)
-      u_norm, p_norm = jax.tree_map(clipping.unitwise_norm, (updates, params))
-      cmp = jax.tree_map(
+      u_norm, p_norm = jax.tree_util.tree_map(
+          clipping.unitwise_norm, (updates, params))
+      cmp = jax.tree_util.tree_map(
           lambda u, p, c=clip_r: u - c * p < 1e-6, u_norm, p_norm)
-      for leaf in jax.tree_leaves(cmp):
+      for leaf in jax.tree_util.tree_leaves(cmp):
         self.assertTrue(leaf.all())
 
       # Check that continuously clipping won't cause numerical issues.
