@@ -117,23 +117,25 @@ def smooth_labels(
 
 
 def sigmoid_binary_cross_entropy(logits, labels):
-  """Computes sigmoid cross entropy given logits and multiple class labels.
+  """Computes element-wise sigmoid cross entropy given logits and labels.
 
-  Measures the probability error in discrete classification tasks in which
-  each class is an independent binary prediction and different classes are
-  not mutually exclusive. This may be used for multilabel image classification
-  for instance a model may predict that an image contains both a cat and a dog.
+  This can be used to measure the error in discrete classification tasks in
+  which each class is an independent binary prediction and different classes
+  are not mutually exclusive. This may be used for multilabel image
+  classification for instance a model may predict that an image contains both a
+  cat and a dog.
 
   References:
     [Goodfellow et al, 2016](http://www.deeplearningbook.org/contents/prob.html)
 
   Args:
-    logits: Unnormalized log probabilities, with shape `[..., num_classes]`.
-    labels: The target probabilities for each class, must have a shape
-        broadcastable to that of `logits`;
+    logits: Each element is the unnormalized log probability of a binary
+      prediction.
+    labels: The target probabilities, must have a shape broadcastable to that of
+      `logits`.
 
   Returns:
-    cross entropy for each binary class prediction, shape `[..., num_classes]`.
+    cross entropy for each binary prediction, same shape as `logits`.
   """
   chex.assert_type([logits], float)
   log_p = jax.nn.log_sigmoid(logits)
@@ -503,3 +505,17 @@ def kl_divergence_with_log_targets(log_predictions: chex.Array,
   chex.assert_type([log_predictions, log_targets], float)
   loss = jnp.exp(log_targets) * (log_targets - log_predictions)
   return jnp.sum(loss, axis=-1)
+
+
+def hinge_loss(predictor_outputs: chex.Array,
+               targets: chex.Array) -> chex.Array:
+  """Computes the hinge loss for binary classification.
+
+  Args:
+    predictor_outputs: Outputs of the decision function.
+    targets: Target values. Target values should be strictly in the set {-1, 1}.
+
+  Returns:
+    Binary Hinge Loss.
+  """
+  return jnp.maximum(0, 1 - predictor_outputs * targets)
