@@ -32,7 +32,7 @@ from optax._src import wrappers
 
 class ConstantTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_constant(self):
     """Check constant schedule."""
     # Get schedule function.
@@ -52,7 +52,7 @@ class ConstantTest(chex.TestCase):
 
 class PolynomialTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_linear(self):
     """Check linear schedule."""
     # Get schedule function.
@@ -69,7 +69,7 @@ class PolynomialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_zero_steps_schedule(self):
     # Get schedule function.
     initial_value = 10.
@@ -83,7 +83,7 @@ class PolynomialTest(chex.TestCase):
       for count in range(15):
         np.testing.assert_allclose(schedule_fn(count), initial_value)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_nonlinear(self):
     """Check non-linear (quadratic) schedule."""
     # Get schedule function.
@@ -102,7 +102,7 @@ class PolynomialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_with_decay_begin(self):
     """Check quadratic schedule with non-zero schedule begin."""
     # Get schedule function.
@@ -126,7 +126,7 @@ class PolynomialTest(chex.TestCase):
 
 class PiecewiseConstantTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_positive(self):
     """Check piecewise constant schedule of positive values."""
     # Get schedule function.
@@ -142,7 +142,7 @@ class PiecewiseConstantTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_negative(self):
     """Check piecewise constant schedule of negative values."""
     # Get schedule function.
@@ -162,7 +162,7 @@ class PiecewiseConstantTest(chex.TestCase):
 
 class ExponentialTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.parameters(False, True)
   def test_constant_schedule(self, staircase):
     """Checks constant schedule for exponential decay schedule."""
@@ -181,7 +181,7 @@ class ExponentialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.parameters(False, True)
   def test_nonvalid_transition_steps(self, staircase):
     """Checks nonvalid decay steps results in a constant schedule."""
@@ -194,7 +194,7 @@ class ExponentialTest(chex.TestCase):
       for count in range(15):
         np.testing.assert_allclose(schedule_fn(count), init_value)
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.parameters(False, True)
   def test_nonvalid_decay_rate(self, staircase):
     """Checks nonvalid decay steps results in a constant schedule."""
@@ -206,7 +206,7 @@ class ExponentialTest(chex.TestCase):
     for count in range(15):
       np.testing.assert_allclose(schedule_fn(count), init_value)
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.parameters((False, 0), (True, 0), (False, 5), (True, 5))
   def test_exponential(self, staircase, transition_begin):
     """Checks non-linear (quadratic) schedule."""
@@ -240,7 +240,7 @@ class ExponentialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.parameters(
       (0.2, 0.1, False), (1.0, 0.1, False), (2.0, 3.0, False),
       (0.2, 0.1, True), (1.0, 0.1, True), (2.0, 3.0, True))
@@ -281,7 +281,7 @@ class ExponentialTest(chex.TestCase):
     np.testing.assert_allclose(
         expected_vals, np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_immutable_count(self):
     """Checks constant schedule for exponential decay schedule."""
     num_steps = 5
@@ -304,7 +304,7 @@ class ExponentialTest(chex.TestCase):
 
 class CosineDecayTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_decay_count_smaller_count(self):
     """Check cosine schedule decay for the entire training schedule."""
     initial_value = 0.1
@@ -324,7 +324,7 @@ class CosineDecayTest(chex.TestCase):
         initial_value * expected_multipliers,
         np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_decay_count_greater_count(self):
     """Check cosine schedule decay for a part of the training schedule."""
     initial_value = 0.1
@@ -345,7 +345,7 @@ class CosineDecayTest(chex.TestCase):
         initial_value * expected_multipliers,
         np.array(generated_vals), atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_decay_count_greater_count_with_alpha(self):
     """Check cosine schedule decay for a part of the training schedule."""
     # Get schedule function.
@@ -371,7 +371,7 @@ class CosineDecayTest(chex.TestCase):
 
 class WarmupCosineDecayTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.named_parameters(
       ('with end value', 10, 0.5, 1e-4),
       ('without end value', 5, 3, 0.),)
@@ -392,7 +392,7 @@ class WarmupCosineDecayTest(chex.TestCase):
 
 class SGDRTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   @parameterized.named_parameters(
       ('with step decay', 1.6, 0.8, 0.4),
       ('without step_decay', 1.6, 1.6, 1.6),)
@@ -410,7 +410,7 @@ class SGDRTest(chex.TestCase):
 
 class PiecewiseInterpolateTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_linear_piecewise(self):
     schedule_fn = self.variant(schedule.piecewise_interpolate_schedule(
         'linear', 200., {5: 1.5, 10: 0.25}))
@@ -419,7 +419,7 @@ class PiecewiseInterpolateTest(chex.TestCase):
                      120., 75., 75., 75.]
     np.testing.assert_allclose(generated_vals, expected_vals, atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_cos_piecewise(self):
     schedule_fn = self.variant(schedule.piecewise_interpolate_schedule(
         'cosine', 400., {5: 1.2, 3: 0.6, 7: 1.}))
@@ -427,7 +427,7 @@ class PiecewiseInterpolateTest(chex.TestCase):
     expected_vals = [400., 360., 280., 240., 264., 288., 288., 288., 288.]
     np.testing.assert_allclose(generated_vals, expected_vals, atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_empty_dict(self):
     schedule_fn = self.variant(schedule.piecewise_interpolate_schedule(
         'linear', 13., {}))
@@ -435,7 +435,7 @@ class PiecewiseInterpolateTest(chex.TestCase):
     expected_vals = [13., 13., 13., 13., 13.]
     np.testing.assert_allclose(generated_vals, expected_vals, atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_no_dict(self):
     schedule_fn = self.variant(schedule.piecewise_interpolate_schedule(
         'cosine', 17.))
@@ -460,7 +460,7 @@ class PiecewiseInterpolateTest(chex.TestCase):
 
 class OneCycleTest(chex.TestCase):
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_linear(self):
     schedule_fn = self.variant(schedule.linear_onecycle_schedule(
         transition_steps=10,
@@ -475,7 +475,7 @@ class OneCycleTest(chex.TestCase):
                      34., 1., 1.]
     np.testing.assert_allclose(generated_vals, expected_vals, atol=1e-3)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_cosine(self):
     schedule_fn = self.variant(schedule.cosine_onecycle_schedule(
         transition_steps=5,
@@ -502,7 +502,7 @@ class InjectHyperparamsTest(chex.TestCase):
   def test_updates(self):
     optim = schedule.inject_hyperparams(transform.scale)(  # stateless
         step_size=schedule.piecewise_constant_schedule(
-            3.0, {2: 5, 8: 2, 13: 1.5}))
+            3.0, {1: 5, 7: 2, 12: 1.5}))
 
     params = [jnp.zeros([], dtype=jnp.float32)]
     state = self.variant(optim.init)(params)
@@ -518,7 +518,7 @@ class InjectHyperparamsTest(chex.TestCase):
   def test_hyperparams_state(self):
     optim = schedule.inject_hyperparams(transform.trace)(  # stateful
         decay=schedule.piecewise_constant_schedule(
-            0.8, {4: 0.5, 10: 1.25}),
+            0.8, {3: 0.5, 9: 1.25}),
         nesterov=True)
 
     params = [jnp.zeros([2, 3]) for _ in range(3)]
@@ -598,11 +598,52 @@ class InjectHyperparamsTest(chex.TestCase):
 
     assert not set(state.hyperparams.keys()).intersection(set(static_args))
 
+  @chex.all_variants
+  @parameterized.named_parameters(
+      ('bf16hyp f32param bf16grad', jnp.bfloat16, jnp.float32, jnp.bfloat16),
+      ('bf16hyp f32param f32_grads', jnp.bfloat16, jnp.float32, jnp.float32),
+      ('f32hyp bf16param bf16grad', jnp.float32, jnp.bfloat16, jnp.bfloat16),
+      ('f32hyp f32param bf16grad', jnp.float32, jnp.float32, jnp.bfloat16),
+      ('f32hyp bf16param f32grad', jnp.float32, jnp.bfloat16, jnp.float32),
+      )
+  def test_hyperparam_dtypes(self,
+                             hyperparam_dtype,
+                             param_dtype,
+                             grad_dtype):
+    """Tests that hyperparam dtype override works as desired."""
+    optim = schedule.inject_hyperparams(
+        transform.scale_by_adam,
+        hyperparam_dtype=hyperparam_dtype)(b1=0.9, b2=0.95)
+
+    params = [jnp.ones((1, 2), dtype=param_dtype),
+              jnp.ones(2, dtype=param_dtype),
+              jnp.ones((1, 1, 1), dtype=param_dtype)]
+    grads = jax.tree_map(lambda x: x.astype(grad_dtype), params)
+    state = self.variant(optim.init)(params)
+    # Check that the hyperparams are overriden
+    self.assertEqual(state.hyperparams['b1'].dtype, hyperparam_dtype)
+    self.assertEqual(state.hyperparams['b2'].dtype, hyperparam_dtype)
+
+    _, state = self.variant(optim.update)(grads, state)
+
+    self.assertEqual(state.hyperparams['b1'].dtype, hyperparam_dtype)
+    self.assertEqual(state.hyperparams['b2'].dtype, hyperparam_dtype)
+
   @parameterized.named_parameters(('string', 'lr'), ('list', ['lr']))
   def test_static_args_error(self, static_args):
     with self.assertRaises(ValueError):
       schedule.inject_hyperparams(transform.scale, static_args=static_args)
 
+  @chex.all_variants
+  def test_inject_hyperparams_starts_with_step_count_zero(self):
+    """Checks that inject_hyperparams uses step count 0 in the first update."""
+    # See also: https://github.com/deepmind/optax/issues/415.
+    opt = schedule.inject_hyperparams(transform.scale)(lambda count: count)
+    params = jnp.zeros(3)
+    grads = jnp.array([-1, 0, 1])
+    updates, _ = self.variant(opt.update)(grads, opt.init(params))
+    np.testing.assert_array_equal(updates, np.zeros(3))
+
 
 if __name__ == '__main__':
   absltest.main()
