@@ -46,16 +46,16 @@ def trace(
 
   Note: `trace` and `ema` have very similar but distinct updates;
   `trace = decay * trace + t`, while `ema = decay * ema + (1-decay) * t`.
-  Both are frequently found in the optimisation literature.
+  Both are frequently found in the optimization literature.
 
   Args:
-    decay: the decay rate for the trace of past updates.
-    nesterov: whether to use Nesterov momentum.
-    accumulator_dtype: optional `dtype` to be used for the accumulator; if
+    decay: Decay rate for the trace of past updates.
+    nesterov: Whether to use Nesterov momentum.
+    accumulator_dtype: Optional `dtype` to be used for the accumulator; if
       `None` then the `dtype` is inferred from `params` and `updates`.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   accumulator_dtype = utils.canonicalize_dtype(accumulator_dtype)
@@ -107,9 +107,17 @@ def update_moment_per_elem_norm(updates, moments, decay, order):
       lambda g, t: (1 - decay) * orderth_norm(g) + decay * t, updates, moments)
 
 
+@functools.partial(jax.jit, inline=True)
 def bias_correction(moment, decay, count):
-  """Perform bias correction. This becomes a no-op as count goes to infinity."""
+  """Performs bias correction. It becomes a no-op as count goes to infinity."""
+  # The conversion to the data type of the moment ensures that bfloat16 remains
+  # bfloat16 in the optimizer state. This conversion has to be done after
+  # `bias_correction_` is calculated as calculating `decay**count` in low
+  # precision can result in it being rounded to 1 and subsequently a
+  # "division by zero" error.
   bias_correction_ = 1 - decay**count
+
+  # Perform division in the original precision.
   return jax.tree_util.tree_map(
       lambda t: t / bias_correction_.astype(t.dtype), moment)
 
@@ -134,16 +142,16 @@ def ema(
 
   Note: `trace` and `ema` have very similar but distinct updates;
   `ema = decay * ema + (1-decay) * t`, while `trace = decay * trace + t`.
-  Both are frequently found in the optimisation literature.
+  Both are frequently found in the optimization literature.
 
   Args:
-    decay: the decay rate for the exponential moving average.
-    debias: whether to debias the transformed gradient.
-    accumulator_dtype: optional `dtype` to used for the accumulator; if `None`
+    decay: Decay rate for the exponential moving average.
+    debias: Whether to debias the transformed gradient.
+    accumulator_dtype: Optional `dtype` to used for the accumulator; if `None`
       then the `dtype` is inferred from `params` and `updates`.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   accumulator_dtype = utils.canonicalize_dtype(accumulator_dtype)
@@ -186,7 +194,7 @@ def scale_by_rss(
     eps: A small floating point value to avoid zero denominator.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -223,12 +231,12 @@ def scale_by_rms(
     [Hinton](www.cs.toronto.edu/~tijmen/csc321/slides/lecture_slides_lec6.pdf)
 
   Args:
-    decay: decay rate for the exponentially weighted average of squared grads.
-    eps: term added to the denominator to improve numerical stability.
-    initial_scale: initial value for second moment
+    decay: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
+    initial_scale: Initial value for second moment.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -263,12 +271,12 @@ def scale_by_stddev(
     [Hinton](www.cs.toronto.edu/~tijmen/csc321/slides/lecture_slides_lec6.pdf)
 
   Args:
-    decay: decay rate for the exponentially weighted average of squared grads.
-    eps: term added to the denominator to improve numerical stability.
-    initial_scale: initial value for second moment
+    decay: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
+    initial_scale: Initial value for second moment.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -309,16 +317,16 @@ def scale_by_adam(
     [Kingma et al, 2014](https://arxiv.org/abs/1412.6980)
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted average of squared grads.
-    eps: term added to the denominator to improve numerical stability.
-    eps_root: term added to the denominator inside the square-root to improve
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
+    eps_root: Term added to the denominator inside the square-root to improve
       numerical stability when backpropagating gradients through the rescaling.
-    mu_dtype: optional `dtype` to be used for the first order accumulator; if
+    mu_dtype: Optional `dtype` to be used for the first order accumulator; if
       `None` then the `dtype is inferred from `params` and `updates`.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   mu_dtype = utils.canonicalize_dtype(mu_dtype)
@@ -344,6 +352,66 @@ def scale_by_adam(
   return base.GradientTransformation(init_fn, update_fn)
 
 
+class ScaleByAmsgradState(NamedTuple):
+  """State for the AMSGrad algorithm."""
+  count: chex.Array  # shape=(), dtype=jnp.int32.
+  mu: base.Updates
+  nu: base.Updates
+  nu_max: base.Updates
+
+
+def scale_by_amsgrad(
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 1e-8,
+    eps_root: float = 0.0,
+    mu_dtype: Optional[Any] = None,
+) -> base.GradientTransformation:
+  """Rescale updates according to the AMSGrad algorithm.
+
+  References:
+    [Reddi et al, 2018](https://openreview.net/forum?id=ryQu7f-RZ)
+
+  Args:
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
+    eps_root: Term added to the denominator inside the square-root to improve
+      numerical stability when backpropagating gradients through the rescaling.
+    mu_dtype: Optional `dtype` to be used for the first order accumulator; if
+      `None` then the `dtype is inferred from `params` and `updates`.
+
+  Returns:
+    A `GradientTransformation` object.
+  """
+
+  mu_dtype = utils.canonicalize_dtype(mu_dtype)
+
+  def init_fn(params):
+    mu = jax.tree_util.tree_map(  # First moment
+        lambda t: jnp.zeros_like(t, dtype=mu_dtype), params)
+    nu = jax.tree_util.tree_map(jnp.zeros_like, params)  # Second moment
+    nu_max = jax.tree_util.tree_map(jnp.zeros_like, params)
+    return ScaleByAmsgradState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu,
+                               nu_max=nu_max)
+
+  def update_fn(updates, state, params=None):
+    del params
+    mu = update_moment(updates, state.mu, b1, 1)
+    nu = update_moment_per_elem_norm(updates, state.nu, b2, 2)
+    count_inc = numerics.safe_int32_increment(state.count)
+    mu_hat = bias_correction(mu, b1, count_inc)
+    nu_hat = bias_correction(nu, b2, count_inc)
+    nu_max = jax.tree_util.tree_map(jnp.maximum, state.nu_max, nu_hat)
+    updates = jax.tree_util.tree_map(
+        lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_max)
+    mu = utils.cast_tree(mu, mu_dtype)
+    return updates, ScaleByAmsgradState(count=count_inc, mu=mu, nu=nu,
+                                        nu_max=nu_max)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
 def scale_by_adamax(
     b1: float = 0.9,
     b2: float = 0.999,
@@ -355,12 +423,12 @@ def scale_by_adamax(
     [Kingma et al, 2014](https://arxiv.org/abs/1412.6980)
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted maximum of grads.
-    eps: term added to the denominator to improve numerical stability.
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted maximum of grads.
+    eps: Term added to the denominator to improve numerical stability.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -375,7 +443,7 @@ def scale_by_adamax(
     nu = update_infinity_moment(updates, state.nu, b2, eps)
     # Bias correction for mean. No bias correction needed for infinity moment.
     mu_hat = bias_correction(mu, b1, count_inc)
-    updates = jax.tree_util.tree_multimap(lambda m, v: m / v, mu_hat, nu)
+    updates = jax.tree_util.tree_map(lambda m, v: m / v, mu_hat, nu)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -390,10 +458,10 @@ def scale(
   """Scale updates by some fixed scalar `step_size`.
 
   Args:
-    step_size: a scalar corresponding to a fixed scaling factor for updates.
+    step_size: A scalar corresponding to a fixed scaling factor for updates.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -417,10 +485,10 @@ def scale_by_param_block_norm(
   (e.g. in a convolutional layer) appearing as a leaf in the grads/param pytree.
 
   Args:
-    min_scale: minimum scaling factor.
+    min_scale: Minimum scaling factor.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -447,10 +515,10 @@ def scale_by_param_block_rms(
   (e.g. in a convolutional layer) appearing as a leaf in the grads/param pytree.
 
   Args:
-    min_scale: minimum scaling factor.
+    min_scale: Minimum scaling factor.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -487,15 +555,15 @@ def scale_by_belief(
     [Zhuang et al, 2020](https://arxiv.org/abs/2010.07468)
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted average of variance of grads.
-    eps: term added to the denominator to improve numerical stability.
-    eps_root: term added to the second moment of the prediction error to
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of variance of grads.
+    eps: Term added to the denominator to improve numerical stability.
+    eps_root: Term added to the second moment of the prediction error to
       improve numerical stability. If backpropagating gradients through the
       gradient transformation (e.g. for meta-learning), this must be non-zero.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -536,16 +604,16 @@ def scale_by_yogi(
     [Zaheer et al, 2018](https://papers.nips.cc/paper/2018/hash/90365351ccc7437a1309dc64e4db32a3-Abstract.html) #pylint:disable=line-too-long
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted average of variance of grads.
-    eps: term added to the denominator to improve numerical stability.
-    eps_root: term added to the denominator inside the square-root to improve
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of variance of grads.
+    eps: Term added to the denominator to improve numerical stability.
+    eps_root: Term added to the denominator inside the square-root to improve
       numerical stability when backpropagating gradients through the rescaling.
     initial_accumulator_value: The starting value for accumulators.
       Only positive values are allowed.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -583,15 +651,15 @@ def scale_by_radam(
     [Liu et al, 2020](https://arxiv.org/abs/1908.03265)
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted average of squared grads.
-    eps: term added to the denominator to improve numerical stability.
-    eps_root: term added to the denominator inside the square-root to improve
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
+    eps_root: Term added to the denominator inside the square-root to improve
       numerical stability when backpropagating gradients through the rescaling.
-    threshold: Threshold for variance tractability
+    threshold: Threshold for variance tractability.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   ro_inf = 2./(1 - b2) - 1
@@ -636,14 +704,14 @@ def add_decayed_weights(
   """Add parameter scaled by `weight_decay`.
 
   Args:
-    weight_decay: a scalar weight decay rate.
-    mask: a tree with same structure as (or a prefix of) the params PyTree,
+    weight_decay: A scalar weight decay rate.
+    mask: A tree with same structure as (or a prefix of) the params PyTree,
       or a Callable that returns such a pytree given the params/updates.
       The leaves should be booleans, `True` for leaves/subtrees you want to
       apply the transformation to, and `False` for those you want to skip.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -676,11 +744,11 @@ def scale_by_schedule(
   """Scale updates using a custom schedule for the `step_size`.
 
   Args:
-    step_size_fn: a function that takes an update count as input and proposes
+    step_size_fn: A function that takes an update count as input and proposes
       the step_size to multiply the updates by.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -718,12 +786,12 @@ def scale_by_trust_ratio(
     [You et. al 2020](https://arxiv.org/abs/1904.00962)
 
   Args:
-    min_norm: minimum norm for params and gradient norms; by default is zero.
-    trust_coefficient: a multiplier for the trust ratio.
-    eps: additive constant added to the denominator for numerical stability.
+    min_norm: Minimum norm for params and gradient norms; by default is zero.
+    trust_coefficient: A multiplier for the trust ratio.
+    eps: Additive constant added to the denominator for numerical stability.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -772,12 +840,12 @@ def add_noise(
     [Neelakantan et al, 2014](https://arxiv.org/abs/1511.06807)
 
   Args:
-    eta: base variance of the gaussian noise added to the gradient.
-    gamma: decay exponent for annealing of the variance.
-    seed: seed for random number generation.
+    eta: Base variance of the gaussian noise added to the gradient.
+    gamma: Decay exponent for annealing of the variance.
+    seed: Seed for random number generation.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -822,10 +890,10 @@ def apply_every(
   important for you, consider using the `optax.MultiSteps`.
 
   Args:
-    k: emit non-zero gradients every k steps, otherwise accumulate them.
+    k: Emit non-zero gradients every k steps, otherwise accumulate them.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -863,7 +931,7 @@ def centralize() -> base.GradientTransformation:
     [Yong et al, 2020](https://arxiv.org/abs/2004.01461)
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def init_fn(params):
@@ -895,12 +963,12 @@ def scale_by_sm3(
     [Anil et. al 2019](https://arxiv.org/abs/1901.11150)
 
   Args:
-    b1: decay rate for the exponentially weighted average of grads.
-    b2: decay rate for the exponentially weighted average of squared grads.
-    eps: term added to the denominator to improve numerical stability.
+    b1: Decay rate for the exponentially weighted average of grads.
+    b2: Decay rate for the exponentially weighted average of squared grads.
+    eps: Term added to the denominator to improve numerical stability.
 
   Returns:
-    An (init_fn, update_fn) tuple.
+    A `GradientTransformation` object.
   """
 
   def zeros_for_dim(p):
@@ -953,6 +1021,117 @@ def scale_by_sm3(
   return base.GradientTransformation(init_fn, update_fn)
 
 
+class ScaleByNovogradState(NamedTuple):
+  """State for Novograd."""
+  count: chex.Array
+  mu: base.Updates
+  nu: base.Updates
+
+
+def scale_by_novograd(
+    b1: float = 0.9,
+    b2: float = 0.25,
+    eps: float = 1e-8,
+    eps_root: float = 0.0,
+    weight_decay: float = 0.0,
+    mu_dtype: Optional[Any] = None,
+) -> base.GradientTransformation:
+  """Computes NovoGrad updates.
+
+  References:
+    [Ginsburg et al, 2019](https://arxiv.org/abs/1905.11286)
+
+  Args:
+    b1: A decay rate for the exponentially weighted average of grads.
+    b2: A decay rate for the exponentially weighted average of squared grads.
+    eps: A term added to the denominator to improve numerical stability.
+    eps_root: A term added to the denominator inside the square-root to improve
+      numerical stability when backpropagating gradients through the rescaling.
+    weight_decay: A scalar weight decay rate.
+    mu_dtype: An optional `dtype` to be used for the first order accumulator; if
+      `None` then the `dtype is inferred from `params` and `updates`.
+
+  Returns:
+    The corresponding `GradientTransformation`.
+  """
+
+  mu_dtype = utils.canonicalize_dtype(mu_dtype)
+
+  def init_fn(params):
+    mu = jax.tree_util.tree_map(  # First moment
+        lambda t: jnp.zeros_like(t, dtype=mu_dtype), params)
+    nu = jax.tree_util.tree_map(lambda _: 0.0, params)  # Second moment
+    return ScaleByNovogradState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
+
+  def nu_addition(grads):
+    return jnp.linalg.norm(grads)**2
+
+  def mu_addition(grads, params, nu):
+    return grads / (jnp.sqrt(nu + eps_root) + eps) + weight_decay * params
+
+  def init_nu(grads, nu):
+    del nu
+    return jax.tree_util.tree_map(nu_addition, grads)
+
+  def update_nu(grads, nu):
+    updates = jax.tree_util.tree_map(nu_addition, grads)
+    return update_moment(updates, nu, b2, 1)
+
+  def init_mu(grads, params, mu, nu):
+    del mu
+    return jax.tree_util.tree_map(mu_addition, grads, params, nu)
+
+  def update_mu(grads, params, mu, nu):
+    updates = jax.tree_util.tree_map(mu_addition, grads, params, nu)
+    return jax.tree_util.tree_map(lambda m, u: b1 * m + u, mu, updates)
+
+  # Second moment
+  def update_fn(updates, state, params):
+    count_inc = numerics.safe_int32_increment(state.count)
+
+    nu = jax.lax.cond(count_inc == 1, init_nu, update_nu, updates, state.nu)
+
+    mu = jax.lax.cond(count_inc == 1, init_mu, update_mu, updates, params,
+                      state.mu, nu)
+
+    mu = utils.cast_tree(mu, mu_dtype)
+    updates = mu
+    return updates, ScaleByNovogradState(count=count_inc, mu=mu, nu=nu)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
+def scale_by_optimistic_gradient(alpha: float = 1.0,
+                                 beta: float = 1.0
+                                ) -> base.GradientTransformation:
+  """Compute generalized optimistic gradients.
+
+  References:
+    [Mokhtari et al, 2019](https://arxiv.org/abs/1901.08511v2)
+
+  Args:
+    alpha: Coefficient for generalized optimistic gradient descent.
+    beta: Coefficient for negative momentum.
+
+  Returns:
+    A `GradientTransformation` object.
+  """
+
+  def init_fn(params):
+    prev_grads = jax.tree_util.tree_map(jnp.zeros_like, params)
+    return TraceState(trace=prev_grads)
+
+  def update_fn(updates, state, params=None):
+    del params
+
+    new_updates = jax.tree_util.tree_map(
+        lambda grad_t, grad_tm1: (alpha + beta) * grad_t - beta * grad_tm1,
+        updates, state.trace)
+    return new_updates, TraceState(trace=updates)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
 # TODO(b/183800387): remove legacy aliases.
 # These legacy aliases are here for checkpoint compatibility
 # To be removed once checkpoints have updated.
