@@ -65,7 +65,7 @@ class TransformTest(parameterized.TestCase):
     jax.tree_util.tree_map(
         lambda *args: chex.assert_equal_shape(args), params, updates)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_add_decayed_weights(self):
     # Define a transform that add decayed weights.
     # We can define a mask either as a pytree, or as a function that
@@ -97,7 +97,7 @@ class TransformTest(parameterized.TestCase):
     # Assert output as expected.
     chex.assert_tree_all_close(new_updates, expected_tx_updates)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_ema(self):
     values = jnp.array([5.0, 7.0])
     decay = 0.9
@@ -115,7 +115,7 @@ class TransformTest(parameterized.TestCase):
         mean,
         (1 - d) * (values[1] + d * values[0]), atol=1e-2)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_ema_debias(self):
     values = jnp.array([5.0, 7.0])
     decay = 0.9
@@ -139,7 +139,7 @@ class TransformTest(parameterized.TestCase):
         (1 - d) * values[1] + d * (1 - d) * values[0],
         atol=1e-2)
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_update_infinity_moment(self):
     values = jnp.array([5.0, 7.0])
     decay = 0.9
@@ -173,7 +173,7 @@ class TransformTest(parameterized.TestCase):
         atol=1e-4
     )
 
-  @chex.all_variants()
+  @chex.all_variants
   def test_apply_every(self):
     # The frequency of the application of sgd
     k = 4
@@ -269,6 +269,37 @@ class TransformTest(parameterized.TestCase):
 
       chex.assert_tree_all_close(updates_i, updates_i_rescaled, rtol=1e-4)
 
+  def test_scale_by_optimistic_gradient(self):
+
+    def f(params: jnp.ndarray) -> jnp.ndarray:
+      return params['x'] ** 2
+
+    initial_params = {
+        'x': jnp.array(2.0)
+    }
+
+    og = transform.scale_by_optimistic_gradient()
+    og_state = og.init(initial_params)
+    # Provide some arbitrary previous gradient.
+    og_state.trace['x'] = 1.5
+
+    g = jax.grad(f)(initial_params)
+    og_true = 2 * g['x'] - og_state.trace['x']
+    og, og_state = og.update(g, og_state)
+
+    # Compare transformation output with manually computed optimistic gradient.
+    chex.assert_tree_all_close(og_true, og['x'])
+
+  @chex.all_variants
+  def test_bias_correction_bf16(self):
+    bias_correction_fn = self.variant(transform.bias_correction)
+    m = jnp.logspace(-10, 10, num=21, dtype=jnp.bfloat16)  # 1e-10 ... 1e10
+    for decay in (0.9, 0.99, 0.999, 0.9995):
+      for count in (1, 10, 100, 1000):
+        chex.assert_tree_all_finite(
+            bias_correction_fn(m, decay, count),
+            custom_message=f'failed with decay={decay}, count={count}')
+
 
 if __name__ == '__main__':
   absltest.main()
